/*
 * main.c:
 *
 * Copyright (c) 2002 James McKenzie <james@fishsoup.dhs.org>,
 * All rights reserved.
 *
 */

static char rcsid[] =
  "$Id: usbradio.c,v 1.3 2002/04/19 11:07:33 root Exp root $";

/*
 * $Log: usbradio.c,v $
 * Revision 1.3  2002/04/19 11:07:33  root
 * #
 *
 * Revision 1.2  2002/04/19 00:31:42  root
 * #
 *
 * Revision 1.1  2002/04/19 00:03:30  root
 * #
 *
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include <usb.h>

#define RADIO_VENDOR  0x4b4
#define RADIO_DEVICE 0x1002

#define RADIO_REQUEST 0xc0
#define TIMEOUT 	20000

#define REQ_STEREO			0
#define STEREO_BIT			1
#define REQ_SET_FREQ			1
#define REQ_MUTE			2
#define MUTED				0
#define UNMUTED				1


#define TEA5757H_FM_IF                          10700000
#define TEA5757H_FREQ_FM_STEP                   12500
#define TEA5757H_FREQ_MASK                      0x7fff

#define BEACON_WAIT_TIME			200000



struct usb_dev_handle *radio = NULL;


static int
usbradio_init ()
{
  struct usb_bus *bus;
  struct usb_device *dev;

  usb_init ();

  usb_find_busses ();
  usb_find_devices ();

  for (bus = usb_busses; bus; bus = bus->next)
    {
      for (dev = bus->devices; dev; dev = dev->next)
        {
          int i;

          if ((dev->descriptor.idVendor == RADIO_VENDOR)
              && (dev->descriptor.idProduct == RADIO_DEVICE))
            {
              radio = usb_open (dev);
              return 0;
            }
        }
    }
  return 1;

}

static uint32_t
freq_to_count (long freq)
{
  freq += TEA5757H_FM_IF;
  freq += (TEA5757H_FREQ_FM_STEP >> 1);
  freq = freq / TEA5757H_FREQ_FM_STEP;
  freq &= TEA5757H_FREQ_MASK;
  return (freq);
}


int
usbradio_stereo (void)
{
  unsigned char buf[7];

  bzero (buf, sizeof (buf));

  if (usb_control_msg
      (radio, RADIO_REQUEST, REQ_STEREO, 0, 0, buf, sizeof (buf),
       TIMEOUT) != 1)
    return -1;

  return (buf[0] & STEREO_BIT) ? 0 : 1;
}


int
usbradio_tune (long freq)       /*Freq in Hz, tuning unmutes, radio will ignore commands for about 30ms after this command */
{
  unsigned char buf[8];
  uint32_t c = freq_to_count (freq);
  int lo = c & 0xffU;
  int hi = (c >> 8U) & 0xffU;

  if (usb_control_msg (radio, RADIO_REQUEST, REQ_SET_FREQ, hi, lo,
                       buf, sizeof (buf), TIMEOUT) != 1)
    return -1;

  return 0;
}


int
usbradio_mute (int i)           /*1=muted, 0=unmuted */
{
  unsigned char buf[8];

  if (usb_control_msg
      (radio, RADIO_REQUEST, REQ_MUTE, i ? MUTED : UNMUTED, 0, buf,
       sizeof (buf), TIMEOUT) != 1)
    return -1;

  return 0;
}

int
main (int argc, char *argv[])
{
  int ret;
  double f;

  if ((argc > 2) && (!strncmp (argv[1], "-h", 2)))
    {
      fprintf (stderr,
               "%s\n"
               "Usage:\n"
               "%s [off|mute|freqency]\n"
               "\n"
               "	Where frequency is a frequency in Hz,kHz or MHz\n",
               "	exit code is the state of the stereo beacon:\n"
               "		1 for stereo, 0 for mono\n", rcsid, argv[0]);
      exit (-1);
    }


  if (usbradio_init ())
    {
      fprintf (stderr, "Can't find radio device\n");
      return -1;
    }

  if ((argc == 2) && (argv[1]))
    {
      f = atof (argv[1]);

      if ((f > 1.0) && (f < 1000.0))
        f *= 1000.0;
      if ((f > 1000.0) && (f < 1000000.0))
        f *= 1000.0;

      if (!strcmp (argv[1], "off"))
        f = 0;
      if (!strcmp (argv[1], "mute"))
        f = 0;


      if (f > 1.0)
        {
          printf ("Tuning to %.2fMHz\n", f / 1000000.0);
          f += 0.5;
          usbradio_tune ((long) f);
        }
      else
        {
          printf ("Mutting audio\n");
          usbradio_mute (1);
        }
      usleep (BEACON_WAIT_TIME);
    }

  ret = usbradio_stereo ();
  printf ("Stereo beacon is %s\n", ret ? "Present" : "Absent");

  return ret;
}
